package com.sankuai.hulk.host.hostmanager.remote;

import com.alibaba.druid.support.json.JSONUtils;
import com.fasterxml.jackson.databind.JsonNode;
import com.google.common.collect.Maps;
import com.sankuai.hulk.host.hostmanager.domain.HulkHost;
import com.sankuai.hulk.host.hostmanager.enums.cluster.EClusterType;
import com.sankuai.hulk.host.hostmanager.exception.InstallApiException;
import com.sankuai.hulk.host.hostmanager.exception.InstallApiExceptionCode;
import com.sankuai.hulk.host.hostmanager.model.cluster.NodeClusterInfo;
import com.sankuai.hulk.host.hostmanager.util.HttpWrapperUtil;
import com.sankuai.hulk.host.hostmanager.util.JsonUtil;
import com.sankuai.hulk.host.hostmanager.util.PropertiesUtil;
import com.sankuai.hulk.host.hostmanager.util.TaskLogWrapper;
import com.sankuai.meituan.config.MtConfigClient;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;
import javax.annotation.Resource;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.util.Strings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

@Service
public class NodeClusterService {

   private static final Logger log = LoggerFactory.getLogger(NodeClusterService.class);
   private String nodeManagerDomainKey = "nodeManager.domain";
   private static final String SEPARATOR = "@";
   private static final String CONNECTOR = ",";
   private static final String DEFAULT_GROUP_NAME = "default";
   @Resource
   private PropertiesUtil propertiesUtil;
   private final MtConfigClient mtConfigClient;


   private String getDomain(String tag) {
      return this.propertiesUtil.getValueByKeyAndTag(tag, this.nodeManagerDomainKey);
   }

   @Autowired
   public NodeClusterService(MtConfigClient mtConfigClient) {
      this.mtConfigClient = mtConfigClient;
   }

   public Integer getNodesByClusterName(String clusterName, String tag) throws Exception {
      log.info("getNodesByClusterName start clusterName={}", clusterName);
      String url = String.format("%s/nodemanager/api/node/list?sn=5&cn=1&clusterName=%s", new Object[]{this.getDomain(tag), clusterName});
      log.info("getNodesByClusterName before, clusterName={} url={}", clusterName, url);
      TaskLogWrapper clusterLogWrapper = HttpWrapperUtil.doGet(url);
      log.info("getNodesByClusterName after, clusterName={} response code={} response={}", new Object[]{clusterName, clusterLogWrapper.getHttpStatusCode(), clusterLogWrapper});
      JsonNode actualResult = (JsonNode)clusterLogWrapper.getActualResult();
      return clusterLogWrapper.getHttpStatusCode().intValue() == 200 && actualResult != null && actualResult.has("code") && actualResult.get("code").asInt() == 0 && actualResult.has("data") && actualResult.get("data").has("tn")?Integer.valueOf(actualResult.get("data").get("tn").asInt()):Integer.valueOf(0);
   }

   public TaskLogWrapper getClusterInfoByName(String clusterName, String tag) throws Exception {
      log.info("NodeClusterService.getClusterInfoByName start clusterName={}", clusterName);
      String url = String.format("%s/api/v1/clusters/%s?withResourceGroup=true", new Object[]{this.getDomain(tag), clusterName});
      log.info("get nodenamager cluster start, clusterName={} url={}", clusterName, url);
      TaskLogWrapper clusterLogWrapper = HttpWrapperUtil.doGet(url);
      log.info("get nodenamager cluster end, clusterName={} response code={} response={}", new Object[]{clusterName, clusterLogWrapper.getHttpStatusCode(), clusterLogWrapper});
      JsonNode actualResult = (JsonNode)clusterLogWrapper.getActualResult();
      if(clusterLogWrapper.getHttpStatusCode().intValue() == 200 && actualResult != null && actualResult.has("code") && actualResult.get("code").asInt() == 0 && actualResult.has("data")) {
         JsonNode clusterInfo = actualResult.path("data");
         NodeClusterInfo response = (NodeClusterInfo)JsonUtil.convert(clusterInfo, NodeClusterInfo.class);
         return new TaskLogWrapper(clusterLogWrapper, response);
      } else {
         return new TaskLogWrapper(clusterLogWrapper, (Object)null);
      }
   }

   public TaskLogWrapper getAllClusters(String isTencent, String isHuawei, String tag) throws Exception {
      String url = String.format("%s/api/v1/clusters/metadata", new Object[]{this.getDomain(tag)});
      HashMap params = Maps.newHashMap();
      if(StringUtils.isNotBlank(isTencent)) {
         params.put("is_tencent", isTencent);
      }

      if(StringUtils.isNotBlank(isHuawei)) {
         params.put("is_huawei", isHuawei);
      }

      String request = "";

      Entry actualResult;
      for(Iterator clusterLogWrapper = params.entrySet().iterator(); clusterLogWrapper.hasNext(); request = request + (String)actualResult.getKey() + "=" + (String)actualResult.getValue()) {
         actualResult = (Entry)clusterLogWrapper.next();
         if(StringUtils.isNotBlank(request)) {
            request = request + "&";
         }
      }

      if(StringUtils.isNotBlank(request)) {
         url = String.format("%s?%s", new Object[]{url, request});
      }

      log.info("getAllClusters start, url={}", url);
      TaskLogWrapper clusterLogWrapper1 = HttpWrapperUtil.doGet(url);
      log.info("getAllClusters end, response code={} response={}", clusterLogWrapper1.getHttpStatusCode(), clusterLogWrapper1);
      JsonNode actualResult1 = (JsonNode)clusterLogWrapper1.getActualResult();
      if(clusterLogWrapper1.getHttpStatusCode().intValue() == 200 && actualResult1.has("code") && actualResult1.get("code").asInt() == 0 && actualResult1.has("data")) {
         JsonNode clusters = actualResult1.path("data");
         if(clusters.isArray()) {
            return new TaskLogWrapper(clusterLogWrapper1, JsonUtil.convertJsonNode2ObjectList(clusters, NodeClusterInfo.class));
         }
      }

      return new TaskLogWrapper(clusterLogWrapper1, (Object)null);
   }

   public TaskLogWrapper getAllClusters(String isTencent, String tag) throws Exception {
      return this.getAllClusters(isTencent, "false", tag);
   }

   public TaskLogWrapper getClusterInfoBySrv(String srv, String tag) throws Exception {
      if(StringUtils.isBlank(srv)) {
         throw new RuntimeException("srv不能为空");
      } else {
         return this.getClusterMeta("srvtree", srv, tag);
      }
   }

   public TaskLogWrapper getClusterMeta(String path, String param, String tag) throws Exception {
      String url = String.format("%s/api/v1/clusters/metadata?%s=%s", new Object[]{this.getDomain(tag), path, param});
      log.info("getClusterInfoBy" + path + " start, url={}", url);
      TaskLogWrapper clusterLogWrapper = HttpWrapperUtil.doGet(url);
      log.info("getClusterInfoBy" + path + " end, response code={} response={}", clusterLogWrapper.getHttpStatusCode(), clusterLogWrapper);
      JsonNode actualResult = (JsonNode)clusterLogWrapper.getActualResult();
      if(clusterLogWrapper.getHttpStatusCode().intValue() == 200 && actualResult.has("code") && actualResult.get("code").asInt() == 0 && actualResult.has("data")) {
         JsonNode nodeCluster = actualResult.path("data");
         if(nodeCluster.isArray()) {
            List nodeClusterInfos = JsonUtil.convertJsonNode2ObjectList(nodeCluster, NodeClusterInfo.class);
            if(CollectionUtils.isNotEmpty(nodeClusterInfos)) {
               return new TaskLogWrapper(clusterLogWrapper, nodeClusterInfos.get(0));
            }
         }
      }

      return new TaskLogWrapper(clusterLogWrapper, (Object)null);
   }

   public boolean isHulkCommon(HulkHost hulkHost) {
      if(ObjectUtils.isEmpty(hulkHost)) {
         return false;
      } else {
         TaskLogWrapper taskLogWrapperNodeCLuster;
         try {
            taskLogWrapperNodeCLuster = this.getClusterInfoByName(hulkHost.getClusterName(), hulkHost.getTag());
         } catch (Exception var9) {
            throw new InstallApiException(InstallApiExceptionCode.NODEMANAGER_CLUSTER_INFO_FAIL.getErrorCode(), "查询调度集群信息失败", var9);
         }

         NodeClusterInfo nodeClusterInfo = (NodeClusterInfo)taskLogWrapperNodeCLuster.getActualResult();
         if(ObjectUtils.isEmpty(nodeClusterInfo)) {
            throw new InstallApiException(InstallApiExceptionCode.NODEMANAGER_CLUSTER_INFO_FAIL.getErrorCode(), "查询调度集群信息失败");
         } else if(!EClusterType.GENERAL.getCode().equals(nodeClusterInfo.getType())) {
            return Boolean.FALSE.booleanValue();
         } else if(Strings.isNotBlank(nodeClusterInfo.getGroupName()) && !"default".equals(nodeClusterInfo.getGroupName())) {
            return Boolean.FALSE.booleanValue();
         } else if(Strings.isBlank(hulkHost.getGroupName()) && Strings.isBlank(hulkHost.getDedicated())) {
            return Boolean.TRUE.booleanValue();
         } else {
            String allowMarkGroupNameStr = this.mtConfigClient.getValue("groupName.computingPower.label");
            allowMarkGroupNameStr = "hulk_pool_buffer,jinrong_hulk,payment_hulk,schedule_pool_buffer";
            String allowNotMarkStr = this.mtConfigClient.getValue("not.allow.computingPower.label");
            Object allowMarkGroupName = new ArrayList();
            Object notAllowMark = new ArrayList();
            if(StringUtils.isNotEmpty(allowMarkGroupNameStr)) {
               allowMarkGroupName = Arrays.asList(allowMarkGroupNameStr.split(","));
            }

            if(StringUtils.isNotEmpty(allowNotMarkStr)) {
               notAllowMark = Arrays.asList(allowNotMarkStr.split(","));
            }

            String curMarkMeta = hulkHost.getGroupName() + "@" + hulkHost.getDedicated();
            log.info("当前装机的机器1,groupName:{},dedicated:{}", hulkHost.getGroupName(), hulkHost.getDedicated());
            if(CollectionUtils.isNotEmpty((Collection)notAllowMark) && ((List)notAllowMark).contains(curMarkMeta)) {
               return Boolean.FALSE.booleanValue();
            } else {
               log.info("lion配置结果:[{}]", allowMarkGroupNameStr);
               log.info("可允许打标签的专区范围是:[{}]", JSONUtils.toJSONString(allowMarkGroupName));
               log.info("判断结果是:[{}]", JSONUtils.toJSONString(Boolean.valueOf(((List)allowMarkGroupName).contains(hulkHost.getGroupName()))));
               if(CollectionUtils.isNotEmpty((Collection)allowMarkGroupName) && ((List)allowMarkGroupName).contains(hulkHost.getGroupName())) {
                  log.info("当前装机的机器2,groupName:{},dedicated:{}", hulkHost.getGroupName(), hulkHost.getDedicated());
                  return Boolean.TRUE.booleanValue();
               } else {
                  return Boolean.FALSE.booleanValue();
               }
            }
         }
      }
   }

}
